/*
 * dcl_cary2obj.c
 *   $Id: dcl_cary2obj.c,v 1.3 2000/11/12 00:37:37 keiko Exp $
 */

#include <math.h>
#include "ruby.h"
#include "f2c.h"
#include "numarray.h"  /* kuro: depend on numarray.so */


/*  functions  */
VALUE dcl_ccharary2obj(char *, int, int);
VALUE dcl_cintegerary2obj(integer *, int, char *);
VALUE dcl_crealary2obj(real *, int, char *);
VALUE dcl_ccomplexary2obj(complex *, int, char *);
VALUE dcl_clogicalary2obj(logical *, int, char *);

static VALUE ccharary2ary(char *, int, int);
static VALUE cintegerary2ary(integer *, int);
static VALUE crealary2ary(real *, int);
static VALUE clogicalary2ary(logical *, int);
/* not inplemented
static VALUE ccomplexary2ary(complex *, int);
*/

/* kuro: depend on numarray.so */
static VALUE crealary2na(real *, int, char *);
static VALUE clogicalary2na(logical *, int, char *);
/* not implemented
static VALUE cintegerary2na(integer *, int, char *);
static VALUE ccomplexary2na(complex *, int, char *);
*/

extern BNA *alloc_bna(etype, size_t);
extern NA *alloc_na(BNA *, int, int *);
extern void free_na(NA *);

/*
 * dcl_cxxxary2obj() : convert c xxx type array to ruby object
 *   dcl_ccharary2obj()
 *   dcl_cintegerary2obj()
 *   dcl_crealary2obj()
 *   dcl_ccomplexary2obj() : not implemented
 *   dcl_clogicalary2obj()
 */
VALUE
dcl_ccharary2obj(cary, size, len)
    char *cary;
    int size;
    int len;
{
    VALUE obj;

    obj = ccharary2ary(cary, size, len);

    return obj;
}

VALUE
dcl_cintegerary2obj(cary, size, type)
    integer *cary;
    int size;
    char *type;
{
    VALUE obj;

    if ((type == NULL) || ( *type == '\0' )) {
	obj = cintegerary2ary(cary, size);
    } else {
	/* not implemented
	obj = cintegerary2na(cary, size, type);
	*/
    }

    return obj;
}

VALUE
dcl_crealary2obj(cary, size, type)
    real *cary;
    int size;
    char *type;
{
    VALUE obj;

    if ((type == NULL) || ( *type == '\0' )) {
	obj = crealary2ary(cary, size);
    } else {
	obj = crealary2na(cary, size, type);
    }

    return obj;
}

/* not implemented
VALUE
dcl_ccomplexary2obj(cary, size, type)
    complex *cary;
    int size;
    char *type;
{
    VALUE obj;

    if ((type == NULL) || ( *type == '\0' )) {
	obj = ccomplexary2ary(cary, size);
    } else {
	obj = ccomplexary2na(cary, size, type);
    }

    return obj;
}
*/

VALUE
dcl_clogicalary2obj(cary, size, type)
    logical *cary;
    int size;
    char *type;
{
    VALUE obj;

    if ((type == NULL) || ( *type == '\0' )) {
	obj = clogicalary2ary(cary, size);
    } else {
	obj = clogicalary2na(cary, size, type);
    }

    return obj;
}


/*
 *  cxxxary2ary() : convert c xxx type array to Array object
 *                  called by cxxxary2obj()
 *   ccharary2ary()
 *   cintegerary2ary()
 *   crealary2ary()
 *   ccomplexary2ary() : not implemented
 *   clogicalary2ary()
 */
static VALUE
ccharary2ary(cary, size, len)
    char *cary;
    int size;
    int len;
{
    VALUE obj, item;
    char *buf, *hd, *ed;
    int i, j;

    if (cary == NULL)
	rb_raise(rb_eRuntimeError, "cannot create Array");

    obj = rb_ary_new2(size/len);
    buf = ALLOCA_N(char, len+1);
    hd = cary;
    for (i = 0; i < size/len; i++) {
        j = len - 1;
        ed = hd + j;
        while ((j >= 0) && (*ed == ' ' || *ed == '\0')) {
            ed--;
            j--;
        }
        if (j >= 0) memcpy(buf, hd, j+1);
        buf[j+1] = '\0';
        item = rb_str_new2(buf);
        rb_ary_push(obj, item);
        hd += len;
    }

    return obj;
}

static VALUE
cintegerary2ary(cary, size)
    integer *cary;
    int size;
{
    VALUE obj, item;
    int i;

    if (cary == NULL)
	rb_raise(rb_eRuntimeError, "cannot create Array");

    obj = rb_ary_new2(size);

    for (i = 0; i < size; i++) {
        item = INT2NUM(cary[i]);
	rb_ary_push(obj, item);
    }

    return obj;
}

static VALUE
crealary2ary(cary, size)
    real *cary;
    int size;
{
    VALUE obj, item;
    int i;

    if (cary == NULL)
	rb_raise(rb_eRuntimeError, "cannot create Array");

    obj = rb_ary_new2(size);

    for (i = 0; i < size; i++) {
	item = rb_float_new((double)cary[i]);
	rb_ary_push(obj, item);
    }

    return obj;
}

/* not inplemented
static VALUE
ccomplexary2ary(cary, size)
    complex *cary;
    int size;
{
    VALUE obj, item;
    int i;

    if (cary == NULL)
	rb_raise(rb_eRuntimeError, "cannot create Array");

    obj = rb_ary_new2(size);

    for (i = 0; i < size; i++) {
	item = ... cary[i];
	rb_ary_push(obj, item);
    }

    return obj;
}
*/

static VALUE
clogicalary2ary(cary, size)
    logical *cary;
    int size;
{
    VALUE obj, item;
    int i;

    if (cary == NULL)
	rb_raise(rb_eRuntimeError, "cannot create Array");

    obj = rb_ary_new2(size);

    for (i = 0; i < size; i++) {
	item = (cary[i] == FALSE_) ? Qfalse : Qtrue;
	rb_ary_push(obj, item);
    }

    return obj;
}


/* kuro: depend on numarray.so */
/*
 *  cxxxary2na() : convert c xxx type array to NumArray object
 *                 called by cxxxary2obj()
 *   cintegerary2na() : not implemented
 *   crealary2na()
 *   ccomplexary2na() : not implemented
 *   clogicalary2na()
 */
static VALUE
crealary2na(cary, size, type)
    real *cary;
    int size;
    char *type;
{
    VALUE obj;
    BNA *bna;
    NA *na;
    int rank, *shape, ofst, i;
    char *wk;
    double *dp;

    if (cary == NULL)
	rb_raise(rb_eRuntimeError, "cannot create NumArray");

    /* type format : "NA:%d:%d,%d...", rank, size[0], size[1] ... */
    if (sscanf(type, "NA:%d:%d%n", &rank, &i, &ofst) == 0)
	rb_raise(rb_eRuntimeError, "cannot create NumArray");

    shape = ALLOCA_N(int, rank);
    shape[0] = i;
    wk = type + ofst;
    for (i = 1; i < rank; i++) {
	sscanf(wk, ",%d%n", &(shape[i]), &ofst);
	wk += ofst;
    }

    bna = alloc_bna(tDOUBLE, size);
    dp = bna->ptr;
    for (i = 0; i < size; i++) {
	dp[i] = (double)cary[i];
    }

    na = alloc_na(bna, rank, shape);

    obj = Data_Wrap_Struct(cNumArrayDouble, 0, free_na, na);

    return obj;
}

static VALUE
clogicalary2na(cary, size, type)
    logical *cary;
    int size;
    char *type;
{
    VALUE obj;
    BNA *bna;
    NA *na;
    int rank, *shape, ofst, i, j, k;
    char *wk;
    unsigned char *up;

    if (cary == NULL)
	rb_raise(rb_eRuntimeError, "cannot create NumArray");

    /* type format : "NA:%d:%d,%d...", rank, size[0], size[1] ... */
    if (sscanf(type, "NA:%d:%d%n", &rank, &i, &ofst) == 0)
	rb_raise(rb_eRuntimeError, "cannot create NumArray");

    shape = ALLOCA_N(int, rank);
    shape[0] = i;
    wk = type + ofst;
    for (i = 1; i < rank; i++) {
	sscanf(wk, ",%d%n", &(shape[i]), &ofst);
	wk += ofst;
    }

    bna = alloc_bna(tLOGICAL, size);
    up = bna->ptr;
    i = 0;
    for (j = 0; j < (size-1)/8; j++) {
        for (k = 128; k > 0; k = k/2) {
            if (cary[i++] == TRUE_) {
                up[j] = (up[j] | k);
            } else {
                /* bna->ptr is 0 cleared */
            }
	    if ( i == size ) break;
	}
    }

    na = alloc_na(bna, rank, shape);

    obj = Data_Wrap_Struct(cNumArrayDouble, 0, free_na, na);

    return obj;
}
